import { Observable, Subject } from 'rxjs';
import { Socket } from 'socket.io-client';
import { AddressStxBalanceResponse, AddressTransactionWithTransfers, Block, MempoolTransaction, Transaction } from '@stacks/stacks-blockchain-api-types';
import { StacksTransaction, TxBroadcastResult } from '@stacks/transactions';
import { StacksNetwork } from '@stacks/network';
export declare const HIRO_API_URL = "https://stacks-node-api.mainnet.stacks.co";
export declare const HIRO_TESTNET_API_URL = "https://stacks-node-api.testnet.stacks.co";
interface RxStacksConfig {
    url: string;
}
declare type AddressTransactionTopic = `address-transaction:${string}`;
declare type AddressStxBalanceTopic = `address-stx-balance:${string}`;
declare type Topic = 'block' | 'mempool' | AddressTransactionTopic | AddressStxBalanceTopic;
interface ClientToServerMessages {
    subscribe: (topic: Topic | Topic[], callback: (error: string | null) => void) => void;
    unsubscribe: (...topic: Topic[]) => void;
}
interface ServerToClientMessages {
    block: (block: Block) => void;
    mempool: (transaction: MempoolTransaction) => void;
    [key: AddressTransactionTopic]: (address: string, stxBalance: AddressTransactionWithTransfers) => void;
    'address-transaction': (address: string, tx: AddressTransactionWithTransfers) => void;
    [key: AddressStxBalanceTopic]: (address: string, stxBalance: AddressStxBalanceResponse) => void;
    'address-stx-balance': (address: string, stxBalance: AddressStxBalanceResponse) => void;
}
export declare class RxStacks {
    config: RxStacksConfig;
    private apiConfig;
    private blocksApi;
    private txApi;
    constructor(config: RxStacksConfig);
    socket$: Observable<Socket<ServerToClientMessages, ClientToServerMessages>>;
    private createEventObservable;
    blocks$: Observable<Block>;
    mempoolTxs$: Observable<MempoolTransaction>;
    txs$: Observable<Transaction>;
    getAddressTransaction(address: string): Observable<AddressTransactionWithTransfers>;
    broadcastTx(tx: StacksTransaction, network: StacksNetwork): Observable<TxBroadcastResult>;
    initialBlockHeight$: Observable<number>;
    currentBlockHeight$: Subject<number>;
    polling: {
        blocksPolling$: Observable<import("@stacks/blockchain-api-client").Block>;
        mempoolTxsPolling$: Observable<import("@stacks/blockchain-api-client").MempoolTransactionListResponse>;
    };
}
export {};
