"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RxStacks = exports.HIRO_TESTNET_API_URL = exports.HIRO_API_URL = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const socket_io_client_1 = require("socket.io-client");
const blockchain_api_client_1 = require("@stacks/blockchain-api-client");
const transactions_1 = require("@stacks/transactions");
exports.HIRO_API_URL = 'https://stacks-node-api.mainnet.stacks.co';
exports.HIRO_TESTNET_API_URL = 'https://stacks-node-api.testnet.stacks.co';
function getWsUrl(url) {
    let urlObj;
    try {
        urlObj = new URL(url);
        if (!urlObj.protocol || !urlObj.host) {
            throw new TypeError(`[ERR_INVALID_URL]: Invalid URL: ${url}`);
        }
    }
    catch (error) {
        console.error(`Pass an absolute URL with a protocol/schema, e.g. "wss://example.com"`);
        throw error;
    }
    return urlObj;
}
class RxStacks {
    constructor(config) {
        this.config = config;
        this.apiConfig = new blockchain_api_client_1.Configuration({ basePath: this.config.url });
        this.blocksApi = new blockchain_api_client_1.BlocksApi(this.apiConfig);
        this.txApi = new blockchain_api_client_1.TransactionsApi(this.apiConfig);
        this.socket$ = new rxjs_1.Observable(subscriber => {
            const socket = socket_io_client_1.io(getWsUrl(this.apiConfig.basePath).href, {
                query: {
                    subscriptions: ['block', 'mempool'].join(','),
                },
            });
            subscriber.next(socket);
            return () => {
                console.log('closing socket');
                socket.close();
            };
        }).pipe(
        // Ensures that socket$ only creates 1 instance
        operators_1.share());
        this.blocks$ = this.createEventObservable('block', 'block', subscriber => block => subscriber.next(block));
        this.mempoolTxs$ = this.createEventObservable('mempool', 'mempool', subscriber => mempool => subscriber.next(mempool));
        this.txs$ = this.blocks$.pipe(operators_1.concatMap(block => Promise.all(block.txs.map(txId => this.txApi.getTransactionById({ txId })))), operators_1.concatMap(arr => rxjs_1.from(arr)));
        this.initialBlockHeight$ = rxjs_1.from(this.blocksApi.getBlockList({ limit: 1 })).pipe(operators_1.map(resp => resp.results[0].height));
        this.currentBlockHeight$ = new rxjs_1.Subject();
        this.polling = {
            blocksPolling$: rxjs_1.merge(this.initialBlockHeight$, this.currentBlockHeight$).pipe(operators_1.concatMap(height => {
                return rxjs_1.timer(0, 20000).pipe(operators_1.tap(() => console.log('Polling for block ', height + 1)), operators_1.concatMap(() => rxjs_1.from(this.blocksApi.getBlockByHeight({ height: height + 1 })).pipe(operators_1.catchError(() => rxjs_1.EMPTY))), operators_1.take(1), operators_1.tap(block => this.currentBlockHeight$.next(block.height)));
            })),
            mempoolTxsPolling$: rxjs_1.from(this.txApi.getMempoolTransactionList({})).pipe(operators_1.tap(initialResults => console.log(initialResults))),
        };
    }
    createEventObservable(topic, eventName, handler) {
        return this.socket$.pipe(operators_1.concatMap(socket => new rxjs_1.Observable(subscriber => {
            socket.emit('subscribe', topic, err => {
                if (err)
                    console.error('error subscribing to', err, topic, eventName);
            });
            socket.on(eventName, handler(subscriber));
        })));
    }
    getAddressTransaction(address) {
        return this.createEventObservable(`address-transaction:${address}`, 'address-transaction', subscriber => (addr, tx) => {
            console.log('getAddressTransaction', address, addr, tx);
            if (address === tx.tx.sender_address)
                subscriber.next(tx);
        });
    }
    broadcastTx(tx, network) {
        return rxjs_1.from(transactions_1.broadcastTransaction(tx, network));
    }
}
exports.RxStacks = RxStacks;
//# sourceMappingURL=main.js.map